;*******************************************************************************************
;   Beschreibung:    7-Segment-Anzeige									       							 *
;							Mit dem Programm wird eine zweistellige 7-Segment-Anzeige mit nur     *
;							9 Ausgangspins angesteuert. Die 7-Segment-Anzeige kann daher direkt   *
;                    mit dem Entwicklungsboard angesteuert werden.                         *
;							In dem Programm wird ein Timer (TMR2) immer wieder neu gestartet und  *
;							lst zyklisch einen Interrupt aus. In der Interruptroutine werden     *
;							dabei abwechselnd die erste und zweite Stelle angesteuert. Da der     *
;							Wechsel zwischen den beiden Stellen im Takt von ca. 4 ms erfolgt,     *
;							kann das Auge den Wechsel nicht wahrnehmen.                           *
;							Parallel zu der Ansteuerung wird ein Zhler in der Hauptschleife      *
;							hochgezhlt. Der Wert in diesem Zhler wird dann auf der Anzeige      *
;							dargestellt.                                                          *
;																														 *
;*******************************************************************************************
;                                                                                          *
;    Dateiname:	 	7-Segment.asm						                                        *
;    Datum:        	20.04.2010                                                            *
;    Version: 			V1.0                                                                  *
;                                                                                          *
;    Autor:       	Michael Hofmann                                                       *
;    Internet:      	www.edmh.de                                                           *
;    e-mail:			info@edmh.de                                                          *
;                                                                                          *
;*******************************************************************************************
; 	  Bemerkungen:                                                                          *
;		Es muss eine 7-Segment-Anzeige mit gemeinsamer Kathode verwendet werden              *
;											                          	                               *
;                                                                                          *
;*******************************************************************************************


	list      p=16f876a            ; definiert den verwendeten Prozessor
	#include <p16f876a.inc>        ; in diesem File stehen die Adressen und Bitnamen
	#include <Macros.inc>			;Makros fr die Bankumschaltung und die Zeitverzgerung
	#include <Zahlen.inc>			;Makros fr die darstellbaren Zahlen/Zeichen auf der 7-Segment Anzeige

	__CONFIG _CP_OFF & _WDT_OFF & _BODEN_OFF & _PWRTE_OFF & _XT_OSC & _WRT_OFF & _LVP_OFF & _DEBUG_ON & _CPD_OFF 
	; Mit '__CONFIG' werden die Configurationsbits gesetzt, z.B. Watchdog an/aus, Debugmodus

;===========================
;  DEFINITIONEN
;===========================
#define SEG_A			PORTB, 0		;Segment A
#define SEG_B			PORTB, 1		;Segment B
#define SEG_C			PORTB, 2		;Segment C
#define SEG_D			PORTB, 3		;Segment D
#define SEG_E			PORTC, 1		;Segment E
#define SEG_F			PORTC, 0		;Segment F
#define SEG_G			PORTA, 5		;Segment G
#define SEG_DP			PORTA, 4		;Segment Dezimalpunkt

#define STELLE_1		PORTA, 2		;erste Dezimalstelle
#define STELLE_2		PORTA, 3		;zweite Dezimalstelle

#define LED_3			PORTB, 4		;LED 3 liegt an Pin RB4
#define LED_4			PORTB, 5		;LED 4 liegt an Pin RB5

;===========================
;  VARIABLEN
;===========================
w_temp        	EQU	0x70        ; werden fr Interrupts verwendet, um den Status vor 
status_temp   	EQU	0x71        ; der Interruptroutine zu sichern
DISP_VALUE		EQU	0x20			; Register welches auf der 7-Seg-Anzeige dargestellt wird
TOGGLE			EQU	0x21			; Toggle-Flag fr die Darstellung der Stelle
COUNTER			EQU	0x22			; Zhl-Register
COPY_REG			EQU	0x23			; Register fr Kopie von DISP_VALUE

;__________________________________________________________________________________________

		ORG     0x000				; Adresse, an die nach einem Reset gesprungen wird
		clrf    PCLATH          ; Rcksetzen der Page-Bits
  		goto    start           ; Springe zum Programmanfang

		ORG     0x004           ; Adresse, an die nach einem Interrupt gesprungen wird
		movwf	  w_temp          ; Sichere den Inhalt des W-Registers
		movf    STATUS,w        ; Verschiebe STATUS-Register ins W-Register
		movwf	  status_temp     ; Sichere den Inhalt des STATUS-Registers

		; Ab hier steht der Code fr die Interruptroutine
		comf TOGGLE,f				;Mit jedem Aufruf der Interruptroutine wird die Stelle gewechselt
		btfss TOGGLE, 0
		goto show_stelle_1
		btfsc TOGGLE, 0
		goto show_stelle_2
show_stelle_1
		swapf DISP_VALUE, F		;obere 4 bits mit den unteren vertauschen, da der obere Wert
										;angezeigt werden soll
		call UpdateSegment		;Der Wert wird auf der 7-Segment-Anzeige angezeigt
		bsf STELLE_1				;Die erste Stelle wird aktiviert
		swapf DISP_VALUE, F		;wieder zurck tauschen
		goto restart_timer
show_stelle_2
		bcf STELLE_1
		bcf STELLE_2
		call UpdateSegment		;Der Wert wird auf der 7-Segment-Anzeige angezeigt
		bsf STELLE_2				;Die zweite Stelle wird aktiviert
		goto restart_timer
restart_timer
		bcf PIR1, TMR2IF			;Timer-berlauf ist aufgetreten, berlaufbit zurcksetzen
		_TMR2_4MS					;Timer neu starten (4 ms)

		movf    status_temp,w     	; hole die Kopie des STATUS-Registers
		movwf	  STATUS            	; Sichere die Daten wieder zurck in das STATUS-Register
		swapf   w_temp,f
		swapf   w_temp,w          	; Sichere die Daten wieder zurck in das W-Register
		retfie                    	; Rcksprung vom Interrupt
;__________________________________________________________________________________________

start							; Beginn des Programms
	; Initialisierungen
	_BANK_0					;Zum Start auf Bank 0 schalten
	clrf PORTA				;alle bits von PORTA auf 0 setzen
	clrf PORTB				;alle bits von PORTB auf 0 setzen
	clrf PORTC				;alle bits von PORTC auf 0 setzen
	_BANK_1					;Umschalten auf Bank 1 fr TRIS Register und ADCON1
	movlw b'00000110'		;alle Pins an PortA sind digitale I/O-Pins
	movwf ADCON1
	movlw b'11000011' 	;Setze RA<1:0> und RA<7:6> als Eingang
	movwf TRISA 			;und setze RA<5:2> als Ausgang
	movlw b'11000000'		;Setze RB<7:6> als Eingang und den Rest als Ausgang
	movwf TRISB
	movlw b'11111100'		;Setze RC<1:0> als Ausgang den Rest als Eingang
	movwf TRISC
	_BANK_0					;Zurckschalten auf Bank 0

	clrf TOGGLE				;TOGGLE-Register auf 0 setzen
	movlw b'11000000'		;Interrups von der Peripherie zulassen
	movwf INTCON

	clrf COUNTER
	clrf DISP_VALUE
	_TMR2_4MS
main								; Beginn der Hauptschleife
	movf COUNTER, W			;Wert aus dem COUNTER-Register kopieren
	movwf DISP_VALUE
	_DELAY_TMR1_US d'500000'	;Der Zhler soll im Takt von 500 ms hochgezhlt werden
	incf COUNTER, F

	goto main					;Fange von vorne an

;__________________________________________________________________________________________

;===========================
;  Unterprogramme
;===========================

;*******************************************************************************************
; Es werden die unteren 4 bits aus dem Register DISP_VALUE auf der 7-Segment-Anzeige
; dargestellt
;*******************************************************************************************
UpdateSegment
	movf DISP_VALUE, W
	andlw 0x0F				;untere Bits maskieren
	movwf COPY_REG			;Register DISP_VALUE kopieren
	
	movlw d'0'
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_0

	movlw d'1'
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_1
	
	movlw d'2'
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_2

	movlw d'3'
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_3

	movlw d'4'
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_4

	movlw d'5'
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_5

	movlw d'6'
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_6

	movlw d'7'
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_7

	movlw d'8'
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_8

	movlw d'9'
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_9

	movlw 0xA
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_A

	movlw 0xB
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_B

	movlw 0xC
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_C

	movlw 0xD
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_D

	movlw 0xE
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_E

	movlw 0xF
	subwf COPY_REG, W
	btfsc STATUS, Z		;prfen, ob das Ergebnis 0 ist
	goto updateSegment_F

	goto updateSegment_End	;Wenn nichts zugetroffen hat

updateSegment_0
	_DISP_0 d'0'				;Es wird eine '0' angezeigt
	goto updateSegment_End
updateSegment_1
	_DISP_1 d'0'				;Es wird eine '1' angezeigt
	goto updateSegment_End
updateSegment_2
	_DISP_2 d'0'				;Es wird eine '2' angezeigt
	goto updateSegment_End
updateSegment_3
	_DISP_3 d'0'				;Es wird eine '3' angezeigt
	goto updateSegment_End
updateSegment_4
	_DISP_4 d'0'				;Es wird eine '4' angezeigt
	goto updateSegment_End
updateSegment_5
	_DISP_5 d'0'				;Es wird eine '5' angezeigt
	goto updateSegment_End
updateSegment_6
	_DISP_6 d'0'				;Es wird eine '6' angezeigt
	goto updateSegment_End
updateSegment_7
	_DISP_7 d'0'				;Es wird eine '7' angezeigt
	goto updateSegment_End
updateSegment_8
	_DISP_8 d'0'				;Es wird eine '8' angezeigt
	goto updateSegment_End
updateSegment_9
	_DISP_9 d'0'				;Es wird eine '9' angezeigt
	goto updateSegment_End
updateSegment_A
	_DISP_A d'0'				;Es wird ein 'A' angezeigt
	goto updateSegment_End
updateSegment_B
	_DISP_B d'0'				;Es wird ein 'B' angezeigt
	goto updateSegment_End
updateSegment_C
	_DISP_C d'0'				;Es wird ein 'C' angezeigt
	goto updateSegment_End
updateSegment_D
	_DISP_D d'0'				;Es wird ein 'D' angezeigt
	goto updateSegment_End
updateSegment_E
	_DISP_E d'0'				;Es wird ein 'E' angezeigt
	goto updateSegment_End
updateSegment_F
	_DISP_F d'0'				;Es wird ein 'F' angezeigt
	goto updateSegment_End


updateSegment_End
	
	return


	END                	; Ende des Programms

